;*********************************************************************************
; Programa: motor_24V.asm
; Programador: Josenir Silvrio
;*********************************************************************************
; Programa para acionamento e controle de motor trifsico.
; Produzir trs fases de corrente alternada a partir de corrente contnua
; usando provisoriamente o PIC16F630, mas no projeto final ser usado o PIC16F676.
;*********************************************************************************

	LIST    P=16F630 ;f=inhx8m
	#include "P16F630.inc"
	__CONFIG 3F84; (oscilador interno, sem WDT, Power Up Timer)

;VARIVEIS DE USURIO A PARTIR DA POSIO 32
TEMPO 		EQU .32
EM_BAIXO 	EQU .33
QUANT 		EQU .34
NUM1  		EQU .35
NUM2  		EQU .36
PWM1  		EQU .37
PWM2  		EQU .38
TMP_T		EQU .39
CONFIG  	EQU .40
FASE1   	EQU .41
FASE2   	EQU .42
FASE3   	EQU .43
PWM_TEMPO   EQU .44


#DEFINE	BANK0	BCF STATUS,RP0	;SETA BANK 0 DE MEMRIA
#DEFINE	BANK1	BSF STATUS,RP0	;SETA BANK 1 DE MEMRIA
#DEFINE	ABAIXO_60HZ	PORTA,0		;LED INDICADOR DE FREQUENCIA ABAIXO DE 60HZ
#DEFINE	ACIMA_60HZ	PORTA,1		;LED INDICADOR DE FREQUENCIA ABAIXO DE 60HZ

#DEFINE	SOBEF1	CONFIG,0		;FLAG INCICADOR DE SUBIDA DA FASE 1
#DEFINE	SOBEF2	CONFIG,1		;FLAG INCICADOR DE SUBIDA DA FASE 1
#DEFINE	SOBEF3	CONFIG,2		;FLAG INCICADOR DE SUBIDA DA FASE 1
#DEFINE	PARADO	CONFIG,3		;FLAG INCICADOR DE SUBIDA DA FASE 1


	ORG	.0	; INICIO DO PROGRAMA
	GOTO INIT

	ORG .4	;ROTINA DE TRATAMENTO DE INTERRUPES
	CLRF PORTC		;DESLIGA TODAS AS FASES
	BCF INTCON,RAIF ;LIMPA O FLAG DE INTERRUPO
	BTFSS PORTA,4
		GOTO DESCE
	BTFSS PORTA,5
		GOTO SOBE
	GOTO FIM_RTI
PARA
	MOVLW B'00000011'	;LIGA OS DOIS LEDS
	MOVWF PORTA
	CALL WAIT
	BTFSS PORTA,3
		GOTO PARA
	GOTO FIM_RTI
SOBE
	MOVFW PWM_TEMPO		;VERIFICA SE PWM_TEMPO CHEGOU EM 3
	XORLW .3
	BTFSC STATUS,Z
		GOTO FIM_RTI
	BSF PORTA,0
	DECF PWM_TEMPO,F
	NOP
	NOP
	CLRWDT
	BTFSS PORTA,5
		GOTO $-2
	GOTO GRAVA_TEMPO

DESCE
	MOVFW PWM_TEMPO			;VERIFICA SE PWM_TEMPO CHEGOU EM 254
	XORLW .254
	BTFSC STATUS,Z
		GOTO FIM_RTI
	BSF PORTA,1
	INCF PWM_TEMPO,F
	NOP
	NOP
	CLRWDT
	BTFSS PORTA,4
		GOTO $-2
GRAVA_TEMPO
	MOVLW .1
	MOVWF NUM1			;NUM1 INDICA A POSIO A SER GRAVADA NA EPROM (PWM_TEMPO => POSIO 1)
	MOVFW PWM_TEMPO			;A VARIAVEL PWM_TEMPO  PASSADA PARA W PARA SER GRAVADA NA EPROM
	CALL GRAVA_EPROM

FIM_RTI
	;BLOCO QUE VERIFICA A FREQUENCIA E FAZ A INDICAO ATRAVS DE DOIS LEDS
	CLRF PORTA
	NOP
	MOVFW PORTA
	NOP
	NOP
	RETFIE

INIT
	BCF STATUS,IRP

	MOVLW TRISC			;CARREGA ENDEREO DE TRISC
	MOVWF FSR
	MOVLW	B'00111111' ;DEFINE TODO O PORTC COMO ENTRADA
	MOVWF	INDF		;GRAVA TRISC

	MOVLW TRISA			;CARREGA ENDEREO DE TRISA
	MOVWF FSR
	MOVLW	B'00111000' ;DEFINE ENTRADAS E SAIDAS DO GPIO
	MOVWF	INDF		;GRAVA TRISA

	MOVLW TRISC			;CARREGA ENDEREO DE TRISC
	MOVWF FSR
	MOVLW	B'00000000' ;DEFINE TODO O PORTC COMO SAIDAS
	MOVWF	INDF		;GRAVA TRISC

	CLRF PORTC			;DESLIGA TODAS AS SAIDAS(LOGICA NORMAL)

	MOVLW	IOCA		;DEFINE OS PINOS DO GPIO QUE TERO INTERRUPO HABILITADA
	MOVWF FSR
	MOVLW	B'00110000' ;APENAS GP4 E GP5 TEM INTERRUPO HABILITADA
	MOVWF	INDF		;GRAVA IOC

	MOVLW	WPUA		;DEFINE OS PINOS DO GPIO QUE TERO PULL UP HABILITADA
	MOVWF FSR
	MOVLW	B'00110000' ;APENAS PORTA4 E 5 TM PULL UP HABILITADO
	MOVWF	INDF		;GRAVA WPUA

	MOVLW	OPTION_REG	;OPTION_REG
	MOVWF FSR
	MOVLW	B'00000000' ;PULL UP HABILITADOS EM OPTION_REG
	MOVWF	INDF		;GRAVA OPTION_REG
	
	MOVLW	B'00000111' ;DESLIGA COMPARADORES DE TENSAO
	MOVWF	CMCON   	;REGISTRADOR CMCON

	MOVLW	B'10001000' ;HABILITA INTERRUPES GERAL-7 E PORTA-3
	MOVWF	INTCON   	;REGISTRADOR INTCON

	CLRF PORTA

	CLRWDT

	;L 1 BYTE NA EPROM E GUARDA EM TMP_T
	MOVLW .1
	CALL LE_EPROM
	MOVWF TMP_T			;SALVA O BYTE LIDO EM TMP_T (VARIAVEL PWM_TEMPO)
	XORLW .255			;COMPARA O BYTE 1 COM 255
	BTFSS STATUS,Z		;SE O VALOR DO BYTE 1 FOR = 255, L PADRO
		GOTO INIT_PARTIDA

PADRAO:
	MOVLW .32			;PADRO PARA FREQUENCIA 60HZ
	MOVWF TMP_T

INIT_PARTIDA

	MOVFW TMP_T
	MOVWF PWM_TEMPO

;SEQUNCIA DE ACIONAMENTO EM 6 PASSOS COM ONDA QUADRADA
INIC_6

	;1)2-3
    ;         ---+++
    ;         123321
	MOVLW B'00001010'
	MOVWF PORTC
	CALL WAIT
	;2)1-3
    ;         ---+++
    ;         123321 
	MOVLW B'00001001'
	MOVWF PORTC
	CALL WAIT
	;3)1-2
    ;         ---+++
    ;         123321 
	MOVLW B'00010001'
	MOVWF PORTC
	CALL WAIT
	;4)3-2
    ;         ---+++
    ;         123321 
	MOVLW B'00010100'
	MOVWF PORTC
	CALL WAIT
	;5)3-1
    ;         ---+++
    ;         123321 
	MOVLW B'00100100'
	MOVWF PORTC
	CALL WAIT
	;6)2-1
    ;         ---+++
    ;         123321 
	MOVLW B'00100010'
	MOVWF PORTC
	CALL WAIT

	GOTO INIC_6

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB-ROTINA WAIT - PARA AGUARDAR TEMPO...
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
WAIT:    				;SE TEMPO = .250 O TEMPO SER DE 1 Segundo a 4Mhz
	CLRWDT
	movfw   PWM_TEMPO  	;PASSA O VALOR DE PWM_TEMPO PARA W
	movwf   NUM1		;PASSA O VALOR DE W PARA NUM1
W1	movlw   .20			;CARREGA 20
	movwf   NUM2		;PASSA 255 PARA NUM2
W2	NOP
	decfsz  NUM2,F		;DECREMENTA NUM2, PULA SE ZERO (REPETE NUM2 VEZES)
	goto 	W2		    ;VAI PARA D2, DECREMENTA NOVAMENTE NUM2
	decfsz  NUM1,F		;DECREMENTA NUM1, PULA SE ZERO
	goto 	W1			;VAI PARA D1
	RETURN


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;A FUNO LE_EPROM RECEBE EM W O ENDEREO A SER LIDO E RETORNA EM W O VALOR LIDO NESTE ENDEREO
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
LE_EPROM:
	CLRWDT
	MOVWF NUM1			;GUARDA EM NUM1 O ENDEREO A SER LIDO
	MOVLW EEADR			;L O ENDEREO DE EEADR
	MOVWF FSR			;GRAVA O ENDEREO DE EEADR NO REGISTRADORDE ACESSO INDIRETO
	MOVFW NUM1			;CARREGA O ENDEREO A SER LIDO EM W
	MOVWF INDF			;GRAVA O ENDEREO A SER LIDO EM EEADR
	MOVLW EECON1		;L ENDEREO DE EECON1 
	MOVWF FSR			;GRAVA ENDERO DE EECON1 NO REGISTRADORDE ACESSO INDIRETO
	BSF INDF,0			;SETA O BIT DE LEITURA (PARA LER O CONTEUDO DO ENDEREO PASSADO EM EEADR)
	MOVLW EEDATA		;L O ENDEREO DE EEDATA
	MOVWF FSR			;GRAVA ENDEREO DE EEDATA NO REGISTRADORDE ACESSO INDIRETO
	MOVFW INDF			;L O CONTEUDO DO BYTE EM EEDATA
	RETURN

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;A FUNO GRAVA_EPROM RECEBE O VALOR A SER GRAVADO EM W E A POSIO A SER GRAVADA EM NUM1
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
GRAVA_EPROM:
	BANK1				;MUDA PARA BANCO 1
	MOVWF EEDATA		;PASSA O VALOR DE W (VALOR QUE VEIO PARA SER GRAVADO) PARA EEDATA
	MOVFW NUM1			;PASSA A POSIO A SER GRAVADA PARA W
	MOVWF EEADR			;PASSA A POSIO A SER GRAVADA PARA EEADR
	BSF EECON1,WREN 	;HABILITA GRAVAO DA EPROM
	MOVLW B'01010101'	;55h (85) SEQUENCIA PARA LIBERAO DA GRAVAO NA EPROM
	MOVWF EECON2 
	MOVLW B'10101010'	;AAh (170)
	MOVWF EECON2 ;
	BSF EECON1,WR 		;INICIA GRAVAO
	BCF EECON1,WREN 	;TERMINA GRAVAO DA EPROM
VE_SE_GRAVOU			;AGUARDA A ESCRITA TERMINAR
	CLRWDT
	BTFSC EECON1,WR
		GOTO VE_SE_GRAVOU	
	BANK0				;VOLTA PARA BANCO 0
	RETURN

END
